<?php

namespace Drupal\track_da_files;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\File\FileSystem;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Http\RequestStack;
use Drupal\Core\StringTranslation\StringTranslationTrait;

class Tdf {

  use StringTranslationTrait;

  public const SCHEME = 'public';

  protected $tdfConfig;

  public function __construct(
    protected AccountInterface $currentUser,
    protected ConfigFactory $config,
    protected Connection $connection,
    protected EntityFieldManagerInterface $entityFieldManager,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected ExtensionPathResolver $extensionPathResolver,
    protected FileSystem $fileSystem,
    protected LoggerChannelFactory $logger,
    protected RequestStack $request
  ) {
    $this->tdfConfig = $this->config->get('track_da_files.settings');
  }

  protected function getTdfRoles() {
   $enabled = $track = $this->tdfConfig->get('roles');
   $roles = $this->tdfConfig->get('specific_roles');
   if (is_array($roles) && $roles > 0) {
     // One or more roles are selected.
     foreach (array_keys($this->currentUser->getRoles()) as $rid) {
       // Is the current user a member of one of these roles?
       if (isset($roles[$rid]) && $rid == $roles[$rid]) {
         // User has a role that should be tracked/excluded from tracking.
         $enabled = !$track;

         break;
       }
     }
   }
   else {
     // No role is selected for tracking, therefore all roles should be tracked.
     $enabled = TRUE;
   }

   return $enabled;
  }

  /**
   * @param $tdf_path
   */
  public function track($tdf_path) {
    $track = $this->getTdfRoles();
    $currentRequest = $this->request->getCurrentRequest();
    $file = $currentRequest->query->get('file');
    $file_path = preg_replace('|^\\/system\\/tdf\\/|', '', $tdf_path);
    if ($file) {
      $uri = self::SCHEME . '://' . $file_path;
      $id = $currentRequest->query->get('id');
      $type = $currentRequest->query->get('type');
      $force = $currentRequest->query->get('force');
      $fields = ['fid', 'filename', 'filemime', 'filesize', 'uri'];
      $fileData = $this->connection->select('file_managed', 'f')
        ->fields('f', $fields)
        ->condition('f.uri', $uri)
        ->execute()
        ->fetchAssoc();
      $fid = $fileData['fid'] ?? NULL;
      $filename = $fileData['filename'] ?? '';
      $filemime = $fileData['filemime'] ?? '';
      $filesize = $fileData['filesize'] ?? '';
      $fileuri = $fileData['uri'] ?? '';
      if (file_exists($uri)) {
        if (!empty($fid) && $track) {
          if (!empty($id) && !empty($type)) {
            $this->registerNewDisplay($uri, $fid, $id, $type);
          }
          else {
            $this->registerNewDisplay($uri, $fid);
          }
        }

        return [
          'uri' => $fileuri,
          'fid' => $fid,
          'filename' => $filename,
          'filemime' => $filemime,
          'filesize' => $filesize,
          'id' => $id,
          'type' => $type,
          'force' => $force,
        ];
      }
    }
  }

  /**
   * Registers new files displays.
   */
  protected function registerNewDisplay($uri, $fid, $id = NULL, $type = NULL) {
    // Get variables to use later.
    $currentRequest = $this->request->getCurrentRequest();
    $account = \Drupal::currentUser();
    $uid = $account->id();
    $ip = $currentRequest->getClientIp();
    $browser = \Drupal::service('browscap')->getBrowser();
    $browser_name = $browser['browser'] ?? NULL;
    $browser_version = $browser['version'] ?? NULL;
    $browser_platform = $browser['platform'] ?? NULL;
    $referer = $currentRequest->headers->get('referer') ?? NULL;
    // We check if path id exists.
    if (!empty($fid)) {
      $result = $this->connection->select('track_da_files_paths', 'p')
        ->fields('p', ['pid'])
        ->condition('p.fid', $fid)
        ->condition('p.path', $uri)
        ->execute()
        ->fetchAssoc();
      $pid = $result['pid'] ?? NULL;
    }
    else {
      $result = $this->connection->select('track_da_files_paths', 'p')
        ->fields('p', ['pid'])
        ->condition('p.path', $uri)
        ->execute()
        ->fetchAssoc();
      $pid = $result['pid'] ?? NULL;
    }
    if (!empty($pid)) {
      // If path is registered we make verifications to avoid duplicated entries.
      $query = $this->connection->select('track_da_files', 't')
        ->fields('t', ['ip', 'pid'])
        ->condition('t.pid', $pid)
        ->groupBy('t.ip')
        ->groupBy('t.pid');
      $query->addExpression('MAX(time)', 'max_time');
      $results = $query->execute()->fetchAssoc();
      $last_display = $results['max_time'] ?? NULL;
      $last_display_ip = $results['ip'] ?? NULL;
      $time = \Drupal::time()->getRequestTime();
      if (isset($last_display)) {
        $last_display_no_flood = $last_display + 10;
      }
      // No duplicated entry.
      // If current viewer ip is the same as last displayer ip.
      // And if time has passed between last display and current time.
      // We can continue.
      if (($ip == $last_display_ip) && (isset($last_display_no_flood) && $last_display_no_flood > $time)) {
        return NULL;
      }
    }
    else {
      // If no path id we register new path.
      $this->connection->insert('track_da_files_paths')
        ->fields([
          'fid' => $fid ?? NULL,
          'pid' => $pid ?? NULL,
          'path' => $uri,
        ])
        ->execute();

      // Then we select path id for our file.
      if (!empty($fid)) {
        $query = $this->connection->select('track_da_files_paths', 'p')
          ->fields('p', ['pid'])
          ->condition('p.fid', $fid)
          ->condition('p.path', $uri);
        $results = $query->execute()->fetchAssoc();
      }
      else {
        $query = $this->connection->select('track_da_files_paths', 'p')
          ->fields('p', ['pid'])
          ->condition('p.path', $uri);
        $results = $query->execute()->fetchAssoc();
      }
      $pid = $results['pid'] ?? NULL;
    }
    $this->connection->insert('track_da_files')
      ->fields([
        'pid' => $pid ?? NULL,
        'ip' => $ip,
        'browser' => $browser_name ?? '',
        'browser_version' => $browser_version ?? '',
        'browser_platform' => $browser_platform ?? '',
        'referer' => $referer ?? '',
        'uid' => $uid,
        'time' => \Drupal::time()->getRequestTime(),
        'id' => $id,
        'type' => $type,
      ])
      ->execute();
  }

}
